package com.ruoyi.ss.transactionBill.service.impl;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.*;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.account.domain.SmAccountVo;
import com.ruoyi.ss.account.service.ISmAccountService;
import com.ruoyi.ss.channel.domain.SmChannel;
import com.ruoyi.ss.channel.service.ISmChannelService;
import com.ruoyi.ss.dashboard.BillCountVo;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.receiveBill.service.ReceiveBillService;
import com.ruoyi.ss.record.time.service.IRecordTimeService;
import com.ruoyi.ss.record.time.service.RecordTimeConverter;
import com.ruoyi.ss.refund.domain.Refund;
import com.ruoyi.ss.refund.domain.RefundVO;
import com.ruoyi.ss.refund.service.RefundConverter;
import com.ruoyi.ss.refund.service.RefundService;
import com.ruoyi.ss.store.domain.StoreVo;
import com.ruoyi.ss.store.service.IStoreService;
import com.ruoyi.ss.suit.domain.SuitVo;
import com.ruoyi.ss.suit.service.SuitService;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.TransactionBillVo;
import com.ruoyi.ss.transactionBill.domain.bo.TransactionBillBO;
import com.ruoyi.ss.transactionBill.domain.dto.BillRefundDTO;
import com.ruoyi.ss.transactionBill.domain.enums.*;
import com.ruoyi.ss.transactionBill.mapper.TransactionBillMapper;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.transactionBill.service.TransactionBillValidator;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.ss.user.domain.enums.UserServiceType;
import com.ruoyi.ss.user.mapper.SmUserMapper;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.ss.wxPay.service.IWxPayService;
import com.ruoyi.system.domain.enums.config.ConfigKey;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.task.bill.BillDelayedManager;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.util.*;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 充值记录Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-02-21
 */
@Service("transactionBillService")
@Slf4j
public class TransactionBillServiceImpl implements TransactionBillService {
    @Autowired
    private TransactionBillMapper transactionBillMapper;

    @Autowired
    private ISmChannelService channelService;

    @Autowired
    private TransactionBillValidator transactionBillValidator;

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private ISmUserService userService;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private BillDelayedManager billDelayedManager;

    @Autowired
    private SmUserMapper smUserMapper;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private ISmAccountService smAccountService;

    @Autowired
    private IStoreService storeService;

    @Autowired
    private SuitService suitService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Value("${sm.withdraw.password.enable}")
    private Boolean smWithdrawPasswordEnable;   // 是否开启提现密码

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IRecordTimeService recordTimeService;

    @Autowired
    private RecordTimeConverter recordTimeConverter;

    @Autowired
    private RefundService refundService;

    @Autowired
    private RefundConverter refundConverter;

    @Autowired
    private ReceiveBillService receiveBillService;

    /**
     * 查询充值记录
     *
     * @param billId 充值记录主键
     * @return 充值记录
     */
    @Override
    public TransactionBillVo selectSmTransactionBillByBillId(Long billId)
    {
        return transactionBillMapper.selectSmTransactionBillByBillId(billId);
    }

    /**
     * 查询充值记录列表
     *
     * @param smTransactionBillQuery 充值记录查询条件
     * @return 充值记录
     */
    @Override
    public List<TransactionBillVo> selectSmTransactionBillList(TransactionBillQuery smTransactionBillQuery)
    {
        return transactionBillMapper.selectSmTransactionBillList(smTransactionBillQuery);
    }

    /**
     * 新增充值记录
     *
     * @param transactionBill 充值记录
     * @return 结果
     */
    @Override
    @Transactional
    public int insertSmTransactionBill(TransactionBill transactionBill)
    {
        transactionBill.setCreateTime(DateUtils.getNowDate());
        transactionBill.setBillNo(String.valueOf(SnowFlakeUtil.newId()));
        return transactionBillMapper.insertSmTransactionBill(transactionBill);
    }

    /**
     * 修改充值记录
     *
     * @param transactionBill 充值记录
     * @return 结果
     */
    @Override
    public int updateSmTransactionBill(TransactionBill transactionBill)
    {
        return transactionBillMapper.updateSmTransactionBill(transactionBill);
    }

    /**
     * 批量删除充值记录
     *
     * @param billIds 需要删除的充值记录主键
     * @return 结果
     */
    @Override
    public int deleteSmTransactionBillByBillIds(Long[] billIds)
    {
        return transactionBillMapper.deleteSmTransactionBillByBillIds(billIds);
    }

    /**
     * 删除充值记录信息
     *
     * @param billId 充值记录主键
     * @return 结果
     */
    @Override
    public int deleteSmTransactionBillByBillId(Long billId)
    {
        return transactionBillMapper.deleteSmTransactionBillByBillId(billId);
    }

    /**
     * 查询账单统计
     * @param dto
     * @return
     */
    @Override
    public List<BillCountVo> selectCount(TransactionBillQuery dto) {
        return transactionBillMapper.selectCount(dto);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String addOrder(TransactionBill data) {
        // 下单
        TransactionBill order = parseToOrder(data);

        // 服务费处理
        this.handleService(order);

        ServiceUtil.assertion(this.insertSmTransactionBill(order) != 1, "下单失败");

        // 加入延时队列，超时取消
        billDelayedManager.add(order.getBillNo(), order.getExpireTime().getTime());

        return order.getBillNo();
    }

    // 处理服务费
    private void handleService(TransactionBill order) {
        SmUserVo mch = userService.selectSmUserByUserId(order.getMchId());
        ServiceUtil.assertion(mch == null, "商户不存在，不允许下单");

        BigDecimal serviceCharge = BigDecimal.ZERO; // 订单服务费
        BigDecimal arrivalAmount = order.getMoney();// 订单商户到账金额

        // 月费商户才出账
        if (UserServiceType.MONTH.getType().equals(mch.getServiceType())) {
            int count = receiveBillService.genBillByMonth(mch, YearMonth.now());
            ServiceUtil.assertion(count == 0, "商户出账失败，请刷新后重试");
        }
        // 其他一律为收取服务费
        else {
            BigDecimal serviceRate = this.getServiceRate(order.getDeviceId(), order.getChannelId());
            serviceCharge = serviceRate.multiply(order.getMoney()).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP);    // 服务费
            arrivalAmount = order.getMoney().subtract(serviceCharge);  // 商户最终到账的金额 = 交易金额 - 服务费
        }
        order.setArrivalAmount(arrivalAmount);
        order.setServiceCharge(serviceCharge);
    }

    // 转换为订单所需的数据
    private TransactionBill parseToOrder(TransactionBill data) {
        // 校验
        ServiceUtil.assertion(transactionBillValidator.preAddOrder(data));

        TransactionBill order = new TransactionBill();
        // 基础信息
        order.setUserId(data.getUserId());
        order.setSuitTime(data.getSuitTime());
        order.setSuitTimeUnit(data.getSuitTimeUnit());
        order.setDeviceNo(data.getDeviceNo());      // 设备编号
        order.setChannelId(data.getChannelId());
        order.setMoney(data.getMoney());
        order.setSuitId(data.getSuitId());
        order.setType(TransactionBillType.RECHARGE.getType());       // 类型：订单
        order.setStatus(TransactionBillStatus.UNPAID.getStatus());   // 状态：未支付
        order.setDeviceRechargeStatus(TransactionBillDeviceRechargeStatus.UN_RECHARGE.getStatus()); // 设备状态：未充值

        // 设备信息
        DeviceVO device = deviceService.selectByDeviceNo(order.getDeviceNo());
        ServiceUtil.assertion(device == null, "设备不存在");
        order.setDeviceId(device.getDeviceId());
        order.setMchId(device.getUserId());    // 到账人ID
        order.setDeviceName(device.getDeviceName());
        order.setDeviceMac(device.getMac());

        // 店铺信息
        order.setStoreId(device.getStoreId());
        StoreVo store = storeService.selectSmStoreById(device.getStoreId());
        if (store != null) {
            order.setStoreName(store.getName());
            order.setStoreAddress(String.format("%s%s%s%s", store.getProvince(), store.getCity(), store.getCounty(), store.getAddress()));
        }

        // 套餐信息
        SuitVo suit = suitService.selectSuitBySuitId(order.getSuitId());
        ServiceUtil.assertion(suit == null, "套餐不存在");
        order.setSuitName(suit.getName());

        // 支付过期时间
        long expireTime = TimeUnit.MILLISECONDS.convert(Constants.BILL_UNPAID_TIMEOUT, Constants.BILL_UNPAID_TIMEUNIT) + System.currentTimeMillis();
        order.setExpireTime(new Date(expireTime));

        // 渠道费用计算
        SmChannel channel = channelService.selectSmChannelByChannelId(order.getChannelId());
        order.setChannelCost(channel.getCostRate().multiply(order.getMoney()).divide(new BigDecimal(100), 2, RoundingMode.HALF_UP));

        return order;
    }

    /**
     * 获取服务费率
     * @param deviceId 设备ID
     * @param channelId 支付渠道
     */
    @Override
    public BigDecimal getServiceRate(Long deviceId, Long channelId) {
        DeviceVO device = deviceService.selectSmDeviceByDeviceId(deviceId);
        if (device != null) {
            // 如果设备有配置服务费率，则返回设备服务费
            if (device.getServiceRate() != null) {
                return device.getServiceRate();
            }
            // 否则查询用户是否配置服务费率
            if (device.getUserId() != null) {
                SmUserVo user = userService.selectSimpleById(device.getUserId());
                if (user != null && user.getServiceRate() != null) {
                    return user.getServiceRate();
                }
            }
        }

        // 返回渠道服务费
        SmChannel channel = channelService.selectSmChannelByChannelId(channelId);
        ServiceUtil.assertion(channel == null, "支付渠道不存在");
        return channel.getServiceRate();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String addWithdraw(TransactionBillBO data) {
        Long userId = data.getUserId();
        // 参数校验
        ServiceUtil.assertion(userId == null, "参数错误：用户id为空");

        try {
            ServiceUtil.assertion(!redisLock.lock(RedisLockKey.ADD_WITHDRAW, userId.toString()), "请勿频繁操作提现");

            // 校验信息是否正确
            // 提现密码
            SmUserVo user = smUserMapper.selectSimpleById(userId);
            if (this.smWithdrawPasswordEnable) {
                ServiceUtil.assertion(StringUtils.isBlank(data.getPayPassword()), "提现密码不允许为空");
                ServiceUtil.assertion(StringUtils.isBlank(user.getPayPassword()), "用户未设置提现密码", 10001);
                ServiceUtil.assertion(!Objects.equals(user.getPayPassword(), SecurityUtils.encryptPassword(data.getPayPassword())), "提现密码错误");
            }

            // 判断该账号是不是本人绑定的提现账号
            TransactionBillPayType payType = TransactionBillPayType.parse(data.getChannelId());
            String accountNo = data.getAccountNo();
            if (TransactionBillPayType.BANK.equals(payType)) {
                ServiceUtil.assertion(accountNo == null, "银行卡账户不允许为空");
                ServiceUtil.assertion(!smAccountService.validateUserHasAccount(userId, accountNo, payType.getAccountType()),
                        "该提现账号未绑定，请重新选择");
            } else if (TransactionBillPayType.WECHAT.equals(payType)){
                SmAccountVo wxAccount = smAccountService.selectUserWxAccount(userId);
                ServiceUtil.assertion(wxAccount == null, "用户不存在微信账户");
                accountNo = wxAccount.getAccountNo();
            } else if (TransactionBillPayType.ALI.equals(payType)) {
                SmAccountVo wxAccount = smAccountService.selectUseAliAccount(userId);
                ServiceUtil.assertion(wxAccount == null, "用户不存在支付宝账户");
                accountNo = wxAccount.getAccountNo();
            }

            // 计算各项金额是否准确
            BigDecimal serviceFeeRate = configService.getBigDecimal(ConfigKey.WITHDRAW_SERVICE_FEE_RATE);
            ServiceUtil.assertion(BigDecimal.ZERO.compareTo(serviceFeeRate) > 0, "手续费不允许小于0");
            BigDecimal serviceRecharge = data.getMoney().multiply(serviceFeeRate); // 服务费
            ServiceUtil.assertion(serviceRecharge.compareTo(data.getServiceCharge()) != 0, "手续费计算不一致，请重新申请");
            BigDecimal arrivalAmount = data.getMoney().subtract(serviceRecharge);   // 到账金额
            ServiceUtil.assertion(arrivalAmount.compareTo(data.getArrivalAmount()) != 0, "到账金额计算不一致，请重新申请");

            // 减少余额，并判断提现金额是否超额（减少的是交易金额）
            userService.subtractBalance(userId, data.getMoney(), "提现");

            // 减少余额后的用户信息
            SmUserVo afterUser = smUserMapper.selectSimpleById(userId);

            // 创建申请订单
            TransactionBill bill = new TransactionBill();
            bill.setUserId(userId);
            bill.setType(TransactionBillType.WITHDRAW.getType());
            bill.setMchId(userId);
            bill.setMoney(data.getMoney());
            bill.setArrivalAmount(arrivalAmount);
            bill.setServiceCharge(serviceRecharge);
            bill.setStatus(TransactionBillStatus.WITHDRAW_APPROVING.getStatus());   // 审核中
            bill.setAfterBalance(afterUser.getBalance());
            bill.setAccountNo(accountNo);
            bill.setChannelId(data.getChannelId());
            ServiceUtil.assertion(this.insertSmTransactionBill(bill) != 1, "提现申请失败");

            return bill.getBillNo();
        } finally {
            redisLock.unlock(RedisLockKey.ADD_WITHDRAW, userId.toString());
        }

    }

    /**
     * 提现审核通过
     * @param billId id
     * @param remark 审核意见
     */
    @Override
    @Transactional
    public boolean passWithdraw(Long billId, String remark) {
        try {
            ServiceUtil.assertion(!redisLock.lock(RedisLockKey.APPROVAL_WITHDRAW, billId.toString()), "该提现申请正在审核，请刷新后重试");

            int updateCount = transactionBillMapper.doPassWithdraw(billId, remark);
            ServiceUtil.assertion(updateCount != 1, "提现审核失败，请刷新后重试");

            return true;
        } finally {
            redisLock.unlock(RedisLockKey.APPROVAL_WITHDRAW, billId.toString());
        }
    }

    @Override
    @Transactional
    public boolean rejectWithdraw(Long billId, String remark) {
        try {
            ServiceUtil.assertion(!redisLock.lock(RedisLockKey.APPROVAL_WITHDRAW, billId.toString()), "该提现申请正在审核，请刷新后重试");

            int updateCount = transactionBillMapper.doRejectWithdraw(billId, remark);
            ServiceUtil.assertion(updateCount != 1, "提现审核失败，请刷新后重试");

            // 返还客户余额
            TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillId(billId);
            ServiceUtil.assertion(bill == null || bill.getUserId() == null, "数据不存在");
            userService.addBalance(bill.getUserId(), bill.getMoney(), "提现驳回");

            return true;
        } finally {
            redisLock.unlock(RedisLockKey.APPROVAL_WITHDRAW, billId.toString());
        }
    }

    /**
     * 提现打款
     * @param billId 提现单id
     */
    @Override
    @Transactional
    public boolean payWithdraw(Long billId) {
        ServiceUtil.assertion(billId == null, "参数错误：提现打款id不允许为空");
        try {
            ServiceUtil.assertion(!redisLock.lock(RedisLockKey.PAY_WITHDRAW, billId.toString()), "该提现申请正在打款，请刷新后重试");

            TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillId(billId);
            ServiceUtil.assertion(bill == null, "单据不存在");

            // 更新为打款中
            int updateCount = transactionBillMapper.doPayWithdraw(billId);
            ServiceUtil.assertion(updateCount != 1, "单据状态发生变化，请刷新后重试");

            // 发起打款
            if (TransactionBillPayType.WECHAT.getType().equals(bill.getChannelId())) {
                InitiateBatchTransferResponse res = wxPayService.payWithdraw(billId);
                log.debug(String.valueOf(res));
            } else {
                throw new ServiceException("其他打款方式正在开发中");
            }

            return true;
        } finally {
            redisLock.unlock(RedisLockKey.PAY_WITHDRAW, billId.toString());
        }
    }

    @Override
    @Transactional
    public void withdrawSuccess(Long billId) {
        // 修改状态
        int updateCount = transactionBillMapper.withdrawSuccess(billId);
        ServiceUtil.assertion(updateCount != 1, "单据状态发生变化，请刷新后重试");
    }

    @Override
    @Transactional
    public void withdrawFailed(Long billId) {
        // 修改状态
        int updateCount = transactionBillMapper.withdrawFailed(billId);
        ServiceUtil.assertion(updateCount != 1, "单据状态发生变化，请刷新后重试");
    }

    /**
     * 更新微信批次明细单号
     * @param billId 提现单号
     * @param transferIds 微信批次明细列表
     */
    @Override
    public boolean updateWxTransferDetailIds(Long billId, List<String> transferIds) {
        ServiceUtil.assertion(billId == null || CollectionUtils.isEmpty(transferIds), "参数错误：提现单号或微信批次明细列表为空");
        TransactionBill bill = new TransactionBill();
        bill.setBillId(billId);
        bill.setTransferIds(transferIds);
        return transactionBillMapper.updateSmTransactionBill(bill) == 1;
    }

    @Override
    public List<BillCountVo> selectLandlordCount(TransactionBillQuery dto) {
        List<BillCountVo> list = this.selectCount(dto);

        // 若startDate与endDate不为空，则将其中间数据补全，不让其为空
        if (dto.getStartDate() != null && dto.getEndDate() != null && Objects.equals(dto.getGroupBy(), TransactionBillGroupBy.create_date.name())) {
            List<BillCountVo> newList = new ArrayList<>();
            Date i = dto.getStartDate();
            while (i.compareTo(dto.getEndDate()) <= 0) {
                Date finalI = i;
                BillCountVo count = list.stream().filter(item -> DateUtils.getYYYY_MM_DD(item.getCreateDate()).equals(DateUtils.getYYYY_MM_DD(finalI))).findFirst().orElse(null);
                if (count == null) {
                    count = new BillCountVo();
                    count.setCreateDate(finalI);
                    count.setServiceCharge(BigDecimal.ZERO);
                    count.setWithdraw(BigDecimal.ZERO);
                    count.setRecharge(BigDecimal.ZERO);
                    count.setCreateYear(DateUtils.getYear(finalI));
                    count.setCreateMonth(DateUtils.getMonth(finalI));
                    count.setCreateDay(DateUtils.getDay(finalI));
                    count.setCreateYearMonth(DateUtils.getYYYY_MM(finalI));
                }
                newList.add(count);
                i = DateUtils.addDays(i,1);
            }

            // 列表排序
            newList.sort(Comparator.comparing(BillCountVo::getCreateDate));
            list = newList;
        }

        return list;
    }

    /**
     * 刷新支付结果
     *
     * @param billId
     * @return
     */
    @Transactional
    public int refreshPayResult(Long billId) {
        return this.refreshPayResult(this.selectSmTransactionBillByBillId(billId));
    }

    @Transactional
    public int refreshPayResult(TransactionBillVo bill) {
        return this.refreshPayResult(Collections.singletonList(bill));
    }

    @Override
    @Transactional
    public void paying(Long billId) {
        ServiceUtil.assertion(billId == null, "订单id不允许为空");
        int updateCount = transactionBillMapper.paying(billId);
        ServiceUtil.assertion(updateCount != 1, "订单状态发生变化，请刷新后重试");
    }

    /**
     * 设备充值
     *
     * @param billId
     */
    @Override
    public boolean rechargeDevice(Long billId) {
        ServiceUtil.assertion(billId == null, "参数错误,billId不允许为空");
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.RECHARGE_DEVICE, billId), "当前设备充值请求过于频繁，请等待");
        try {
            TransactionBillVo bill = transactionBillMapper.selectSmTransactionBillByBillId(billId);
            ServiceUtil.assertion(bill == null || !TransactionBillType.RECHARGE.getType().equals(bill.getType()), "不存在的充值订单");
            ServiceUtil.assertion(!TransactionBillStatus.SUCCESS.getStatus().equals(bill.getStatus()), "订单未支付");
            ServiceUtil.assertion(TransactionBillDeviceRechargeStatus.SUCCESS.getStatus().equals(bill.getDeviceRechargeStatus()), "设备已充值成功，不允许再次充值");
            ServiceUtil.assertion(TransactionBillDeviceRechargeStatus.BLUETOOTH.getStatus().equals(bill.getDeviceRechargeStatus()), "设备已选择蓝牙充值，请使用蓝牙进行充值");

            Boolean result = transactionTemplate.execute(status -> {
                // 更新套餐生效时间
                DeviceVO device = deviceService.selectSmDeviceByDeviceId(bill.getDeviceId());
                this.updateSuitTimeBeforeDevice(billId, device.getExpireTime());

                // 修改设备充值状态：成功
                int updateRecharge = transactionBillMapper.updateDeviceRechargeStatus(bill.getBillId(), TransactionBillDeviceRechargeStatus.SUCCESS.getStatus());
                ServiceUtil.assertion(updateRecharge != 1, "订单状态发生变化，请稍后重试");

                try {
                    // 设备充值
                    return deviceService.addTime(bill.getDeviceId(), bill.toSecondSuitTime(), true);
                } catch (Exception e) {
                    // 失败，回滚
                    status.setRollbackOnly();
                    return false;
                }
            });

            // 是否成功
            boolean success = result != null && result;

            if (success) {
                // 记录时长变化
                this.recordTime(bill, "用户充值");
            } else {
                // 修改设备充值状态为失败
                transactionBillMapper.updateDeviceRechargeStatus(billId, TransactionBillDeviceRechargeStatus.FAIL.getStatus());
            }

            return success;
        } finally {
            redisLock.unlock(RedisLockKey.RECHARGE_DEVICE, billId);
        }
    }

    /**
     * 在设备充值成功前，更新套餐时间
     * 开始时间 = 设备过期时间 - 套餐时间，过期时间 = 设备过期时间
     */
    private int updateSuitTimeBeforeDevice(Long billId, LocalDateTime deviceExpireTime) {
        TransactionBillVo bill = this.selectSmTransactionBillByBillId(billId);
        if (bill == null) {
            return 0;
        }
        // 计算开始使用的时间
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime startTime = deviceExpireTime == null || now.isAfter(deviceExpireTime) ? now : deviceExpireTime;

        LocalDateTime endTime = startTime.plusSeconds(bill.toSecondSuitTime());
        TransactionBill data = new TransactionBill();
        data.setBillId(billId);
        data.setSuitStartTime(startTime);
        data.setSuitEndTime(endTime);
        data.setSuitExpireTime(endTime);
        return transactionBillMapper.updateSmTransactionBill(data);
    }

    /**
     * 获取支付结果
     * @param billNo 订单编号
     * @return 支付结果
     */
    @Override
    public boolean getPayResult(String billNo) {
        // 查询订单
        TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        ServiceUtil.assertion(bill == null, "订单不存在");

        if (TransactionBillPayType.WECHAT.getType().equals(bill.getChannelId())) {
            // 微信支付
            Transaction transaction = wxPayService.queryOrderByOutTradeNo(billNo);
            return wxPayService.isSuccess(transaction);
        }
        // ... TODO 其他支付方式

        return false;
    }

    /**
     * 充值成功
     * @param billNo 充值订单编号
     */
    @Override
    public void rechargeSuccess(String billNo, Date payTime) {
        log.info("充值成功,billNo:{}", billNo);
        TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        ServiceUtil.assertion(bill == null || !TransactionBillType.RECHARGE.getType().equals(bill.getType()), "充值订单不存在");

        Integer result = transactionTemplate.execute(status -> {
            // 修改订单状态
            int updateCount = transactionBillMapper.rechargeSuccess(bill.getBillId(), payTime);
            ServiceUtil.assertion(updateCount != 1, "修改订单状态失败，请刷新后重试");

            // 商户余额增加
            userService.addBalance(bill.getMchId(), bill.getArrivalAmount(), String.format("订单充值:%s", billNo));

            // 记录下充值后的余额
            SmUserVo user = userService.selectSmUserByUserId(bill.getMchId());
            updateAfterBalance(bill.getBillId(), user.getBalance());

            return updateCount;
        });

        if (result != null && result == 1) {
            // 设备充值
            rechargeDevice(bill.getBillId());
        }
    }

    /**
     * 记录下操作后的余额
     * @param billId 订单id
     * @param balance 余额
     */
    private void updateAfterBalance(Long billId, BigDecimal balance) {
        TransactionBill data = new TransactionBill();
        data.setBillId(billId);
        data.setAfterBalance(balance);
        transactionBillMapper.updateSmTransactionBill(data);
    }

    /**
     * 取消充值订单
     * @param billNo 订单编号
     * @param status 取消后的状态
     */
    @Override
    @Transactional
    public int cancelRecharge(String billNo, TransactionBillStatus status) {
        ServiceUtil.assertion(!TransactionBillStatus.isCancel(status.getStatus()), "参数错误：状态并非取消状态");

        TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        ServiceUtil.assertion(bill == null, "订单不存在");
        // 用户自行取消，验证身份
        ServiceUtil.assertion(TransactionBillStatus.CANCELED.equals(status) && !UserUtil.hasFrontUser(bill.getUserId()), "这不是您的订单");

        List<String> allowCancel = Arrays.asList(TransactionBillStatus.UNPAID.getStatus(), TransactionBillStatus.PAYING.getStatus());
        ServiceUtil.assertion(!allowCancel.contains(bill.getStatus()), "订单状态非未支付或支付中，不允许取消");
        ServiceUtil.assertion(!TransactionBillType.RECHARGE.getType().equals(bill.getType()), "非充值订单，不允许取消");

        // 执行取消订单
        int updateCount = transactionBillMapper.cancelRecharge(billNo, status.getStatus());
        ServiceUtil.assertion(updateCount != 1, "订单取消失败，请刷新后重试");

        // 关闭支付订单
        if (TransactionBillStatus.PAYING.getStatus().equals(bill.getStatus())) {
            this.closeOrder(billNo);
        }

        return updateCount;
    }

    @Override
    public TransactionBillVo selectSmTransactionBillByBillNo(String billNo) {
        return transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
    }

    @Override
    public List<TransactionBillVo> selectNoPayedRecharge() {
        TransactionBillQuery dto = new TransactionBillQuery();
        dto.setStatusList(Arrays.asList(TransactionBillStatus.UNPAID.getStatus(), TransactionBillStatus.PAYING.getStatus()));
        dto.setType(TransactionBillType.RECHARGE.getType());
        return this.selectSmTransactionBillList(dto);
    }

    /**
     * 查询设备充值失败列表
     *
     * @param userId
     * @return
     */
    @Override
    public List<TransactionBillVo> selectDeviceRechargeFailList(Long userId) {
        if (userId == null) {
            return Collections.emptyList();
        }
        // 查询充值失败列表
        TransactionBillQuery dto = new TransactionBillQuery();
        dto.setUserId(userId);
        dto.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        dto.setDeviceRechargeStatus(TransactionBillDeviceRechargeStatus.FAIL.getStatus());
        dto.setType(TransactionBillType.RECHARGE.getType());
        List<TransactionBillVo> list = this.selectSmTransactionBillList(dto);

        if (!CollectionUtils.isEmptyElement(list)) {
            // 修改状态为蓝牙充值
            transactionBillMapper.updateDeviceRechargeStatusByIds(
                    list.stream().map(TransactionBillVo::getBillId).collect(Collectors.toList()),
                    TransactionBillDeviceRechargeStatus.BLUETOOTH.getStatus());
        }

        // 查询蓝牙充值列表
        dto.setDeviceRechargeStatus(TransactionBillDeviceRechargeStatus.BLUETOOTH.getStatus());
        return selectSmTransactionBillList(dto);
    }


    /**
     * 查询支付结果，直到过期
     *
     * @param billNo     订单编号
     * @param delay      延迟时间
     * @param timeUnit   时间单位
     */
    @Override
    public void refreshPayResultBeforeExpire(String billNo, int delay, TimeUnit timeUnit) {
        TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        if (bill == null) {
            log.error("获取订单{}为空", billNo);
            return;
        }
        Date now = new Date();
        boolean payResult = false;

        // 判断是否支付成功
        try {
            if (TransactionBillStatus.PAYING.getStatus().equals(bill.getStatus())) {
                payResult = getPayResult(bill.getBillNo());
                if (payResult) {
                    // 支付成功
                    this.rechargeSuccess(bill.getBillNo(), now);
                    return;
                }
            }
        } catch (Exception e) {
            log.error("刷新支付结果出错：{}", e.getMessage());
        }

        // 未支付成功
        if (!payResult) {
            if (bill.getExpireTime() != null && now.compareTo(bill.getExpireTime()) < 0) {
                // 未过期，继续延迟查询
                log.info("{}支付失败，未过期，继续延迟查询", billNo);
                scheduledExecutorService.schedule(() -> {
                    this.refreshPayResultBeforeExpire(billNo, delay , timeUnit);
                }, delay, timeUnit);
            } else {
                log.info("{}支付失败，过期取消订单", billNo);
                // 过期取消订单
                int updateCount = transactionBillMapper.cancelRecharge(billNo, TransactionBillStatus.SYS_CANCELED.getStatus());
                log.info("过期取消订单，结果：{}", updateCount);
                if (updateCount == 1) {
                    // 关闭支付订单
                    this.closeOrder(billNo);
                }
            }
        }
    }

    /**
     * 查询数量并映射
     *
     * @param query     查询条件
     * @param keyMapper 映射方法
     */
    @Override
    public <K> Map<K, BillCountVo> selectCountMap(TransactionBillQuery query, Function<? super BillCountVo, ? extends K> keyMapper) {
        List<BillCountVo> list = this.selectCount(query);
        if (CollectionUtils.isEmptyElement(list)) {
            return Collections.emptyMap();
        }
        return list.stream().collect(Collectors.toMap(keyMapper, item -> item));
    }

    // 关闭订单
    private void closeOrder(String billNo) {
        TransactionBill bill = transactionBillMapper.selectSmTransactionBillByBillNo(billNo);
        if (bill == null) {
            return;
        }

        if (TransactionBillPayType.WECHAT.getType().equals(bill.getChannelId())) {
            wxPayService.closeOrder(billNo);
        }
    }


    /**
     * 蓝牙充值成功
     *
     * @param billNo
     * @return
     */
    @Override
    public boolean bluetoothRechargeSuccess(String billNo) {
        TransactionBillVo bill = selectSmTransactionBillByBillNo(billNo);
        ServiceUtil.assertion(bill == null, "订单不存在", 10001);
        ServiceUtil.assertion(!TransactionBillType.RECHARGE.getType().equals(bill.getType()), "该订单不是充值订单", 10002);
        ServiceUtil.assertion(TransactionBillDeviceRechargeStatus.SUCCESS.getStatus().equals(bill.getDeviceRechargeStatus()), "设备已充值成功，请勿重复充值", 10003);
        ServiceUtil.assertion(!TransactionBillStatus.SUCCESS.getStatus().equals(bill.getStatus()), "订单尚未充值成功", 10004);

        Boolean execute = transactionTemplate.execute(status -> {
            // 更新设备充值状态
            boolean result = transactionBillMapper.bluetoothRechargeSuccess(billNo) == 1;
            ServiceUtil.assertion(!result, "蓝牙充值回调失败");

            // 更新套餐时间
            DeviceVO afterDevice = deviceService.selectSmDeviceByDeviceId(bill.getDeviceId());
            this.updateSuitTimeBeforeDevice(bill.getBillId(), afterDevice.getExpireTime());

            // 修改剩余时间失败
            boolean addTime = deviceService.addTime(bill.getDeviceId(), bill.toSecondSuitTime(), false);
            ServiceUtil.assertion(!addTime, "修改剩余时间失败");

            return result;
        });

        // 是否成功
        boolean success = execute != null && execute;
        // 成功后操作
        if (success) {
            // 时长变化记录
            this.recordTime(bill, "蓝牙手动充值");
        }

        return success;
    }

    // 记录时长变化记录
    private void recordTime(TransactionBillVo bill, String reason) {
        scheduledExecutorService.schedule(() -> {
            DeviceVO device = deviceService.selectSmDeviceByDeviceId(bill.getDeviceId());
            if (device == null) {
                return;
            }
            SmUserVo user = userService.selectSmUserByUserId(bill.getUserId());
            recordTimeService.insertRecordTime(recordTimeConverter.toRecordTime(device, bill.toSecondSuitTime(), reason, user));
        }, 1L, TimeUnit.SECONDS);
    }

    @Override
    public int refund(BillRefundDTO dto) {
        // 校验订单
        TransactionBillVo bill = this.selectSmTransactionBillByBillId(dto.getBillId());
        if (bill == null) {
            return 0;
        }
        ServiceUtil.assertion(dto.getRefundAmount().compareTo(bill.getMoney()) > 0, "退款金额不允许大于订单金额");
        ServiceUtil.assertion(!TransactionBillStatus.SUCCESS.getStatus().equals(bill.getStatus()), "当前订单状态不允许退款");

        Integer result = transactionTemplate.execute(status -> {
            // 修改订单状态
            TransactionBill data = new TransactionBill();
            data.setStatus(TransactionBillStatus.REFUNDING.getStatus());
            TransactionBillQuery billQuery = new TransactionBillQuery();
            billQuery.setBillId(dto.getBillId());
            billQuery.setStatus(TransactionBillStatus.SUCCESS.getStatus());
            int updateBill = this.updateByQuery(data, billQuery);
            ServiceUtil.assertion(updateBill != 1, "修改订单状态失败");

            // 创建退款订单
            Refund refund = refundConverter.toPo(dto);
            int updateRefund = refundService.insertRefund(refund);
            ServiceUtil.assertion(updateRefund != 1, "创建退款订单失败");
            RefundVO refundVO = refundService.selectRefundByRefundNo(refund.getRefundNo());

            // 商户余额按照比例扣减
            userService.subtractBalance(bill.getMchId(), refund.getMchAmount(), "退款");

            // 修改原订单的退款金额和退款手续费
            int updateRefundAmount = this.addRefundAmount(bill.getBillId(), refund.getAmount(), refund.getMchAmount(), refund.getServiceAmount());
            ServiceUtil.assertion(updateRefundAmount != 1, "修改原订单的退款金额和退款手续费失败");

            // 发起退款
            if (TransactionBillPayType.WECHAT.getType().equals(bill.getChannelId())) {
                wxPayService.refund(refundVO);
            } else {
                throw new ServiceException("当前支付方式不支持退款");
            }
            return 1;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int updateByQuery(TransactionBill data, TransactionBillQuery query) {
        if (query == null) {
            return 0;
        }
        return transactionBillMapper.updateByQuery(data, query);
    }

    @Override
    @Transactional
    public int refreshPayResult(String billNo) {
        return this.refreshPayResult(this.selectSmTransactionBillByBillNo(billNo));
    }

    @Override
    public int refreshPayResult(List<TransactionBillVo> billList) {
        if (CollectionUtils.isEmptyElement(billList)) {
            return 0;
        }

        // 获取支付结果，并判断是否支付成功，若成功则更新数据
        Date now = new Date();
        for (TransactionBillVo bill : billList) {
            if (TransactionBillStatus.PAYING.getStatus().equals(bill.getStatus()) || TransactionBillStatus.UNPAID.getStatus().equals(bill.getStatus())  ) {
                boolean payResult = getPayResult(bill.getBillNo());
                if (payResult) {
                    this.rechargeSuccess(bill.getBillNo(), now);
                }
            }
        }
        return 1;
    }

    @Override
    public int addRefundAmount(Long billId, BigDecimal refundAmount, BigDecimal refundMchAmount, BigDecimal refundServiceAmount) {
        if (billId == null) {
            return 0;
        }
        return transactionBillMapper.addRefundAmount(billId, refundAmount, refundMchAmount, refundServiceAmount);
    }

    @Override
    public List<TransactionBillVo> selectUsingBill(Long userId) {
        TransactionBillQuery query = new TransactionBillQuery();
        query.setUserId(userId);
        return this.selectUsingBill(query);
    }

    @Override
    public List<TransactionBillVo> selectUsingBill(TransactionBillQuery query) {
        query.setUsing(true);
        query.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        return this.selectSmTransactionBillList(query);
    }

    @Override
    public int batchEndBillByDevice(Long deviceId) {
        if (deviceId == null) {
            return 0;
        }
        LocalDateTime now = LocalDateTime.now();
        TransactionBillQuery query = new TransactionBillQuery();
        query.setDeviceId(deviceId);
        query.setEndSuitEndTime(now);   // 套餐结束时间在这之后的，都会被关闭
        List<TransactionBillVo> billList = this.selectSmTransactionBillList(query);
        if (CollectionUtils.isNotEmptyElement(billList)) {
            return transactionBillMapper.batchUpdateSuitEndTime(billList.stream().map(TransactionBill::getBillId).collect(Collectors.toList()), now);
        }
        return 0;
    }
}
