package com.ruoyi.web.controller.ss;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.ruoyi.common.core.domain.ValidGroup;
import com.ruoyi.ss.device.domain.DeviceBO;
import com.ruoyi.ss.device.domain.dto.DeviceBatchUpdateModelDTO;
import com.ruoyi.ss.device.service.DeviceAssembler;
import com.ruoyi.ss.device.domain.DeviceQuery;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 设备Controller
 *
 */
@RestController
@RequestMapping("/system/device")
public class SmDeviceController extends BaseController
{
    @Autowired
    private DeviceService smDeviceService;

    @Autowired
    private DeviceAssembler deviceAssembler;

    /**
     * 查询设备列表
     */
    @PreAuthorize("@ss.hasPermi('system:device:list')")
    @GetMapping("/list")
    public TableDataInfo list(DeviceQuery smDevice)
    {
        startPage();
        List<DeviceVO> list = smDeviceService.selectSmDeviceList(smDevice);
        deviceAssembler.assembleIotDeviceInfo(list);
        return getDataTable(list);
    }


    /**
     * ids查询设备列表
     */
    @PreAuthorize("@ss.hasPermi('system:device:list')")
    @GetMapping("/listByIds/{ids}")
    public AjaxResult listByIds(@PathVariable Long[] ids)
    {
        List<DeviceVO> list = smDeviceService.selectListByIds(Arrays.asList(ids));
        return success(list);
    }

    /**
     * 导出设备列表
     */
    @PreAuthorize("@ss.hasPermi('system:device:export')")
    @Log(title = "设备", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, DeviceQuery smDevice)
    {
        List<DeviceVO> list = smDeviceService.selectSmDeviceList(smDevice);
        ExcelUtil<DeviceVO> util = new ExcelUtil<DeviceVO>(DeviceVO.class);
        util.exportExcel(response, list, "设备数据");
    }

    /**
     * 获取设备详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:device:query')")
    @GetMapping(value = "/{deviceId}")
    public AjaxResult getInfo(@PathVariable("deviceId") Long deviceId)
    {
        smDeviceService.pullDeviceInfo(Collections.singletonList(deviceId));
        DeviceVO device = smDeviceService.selectSmDeviceByDeviceId(deviceId);
        List<DeviceVO> list = Collections.singletonList(device);
        deviceAssembler.assembleIotDeviceInfo(list);
        return success(device);
    }

    /**
     * 新增设备
     */
    @PreAuthorize("@ss.hasPermi('system:device:add')")
    @Log(title = "设备", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody @Validated({ValidGroup.Create.class}) DeviceBO data)
    {
        data = data.filterCreate();
        return toAjax(smDeviceService.insertSmDevice(data));
    }

    /**
     * 修改设备
     */
    @PreAuthorize("@ss.hasPermi('system:device:edit')")
    @Log(title = "设备", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody @Validated({ValidGroup.Update.class}) DeviceBO data)
    {
        data = data.filterUpdate();
        return toAjax(smDeviceService.updateSmDevice(data));
    }

    /**
     * 删除设备
     */
    @PreAuthorize("@ss.hasPermi('system:device:remove')")
    @Log(title = "设备", businessType = BusinessType.DELETE)
	@DeleteMapping("/{deviceIds}")
    public AjaxResult remove(@PathVariable Long[] deviceIds)
    {
        return toAjax(smDeviceService.deleteSmDeviceByDeviceIds(deviceIds));
    }

    @ApiOperation("逻辑删除设备")
    @PreAuthorize("@ss.hasPermi('system:device:remove')")
    @Log(title = "设备", businessType = BusinessType.LOGIC_DELETE)
    @DeleteMapping("/logic/{deviceIds}")
    public AjaxResult logicDel(@PathVariable Long[] deviceIds)
    {
        return toAjax(smDeviceService.logicDel(Arrays.asList(deviceIds)));
    }

    @ApiOperation("刷新设备信息")
    @GetMapping("/{deviceId}/refreshIot")
    public AjaxResult syncIot(@PathVariable @ApiParam("设备id") Long deviceId) {
        smDeviceService.pullDeviceInfo(Collections.singletonList(deviceId));
        return success();
    }

    @ApiOperation("绑定SN码")
    @PreAuthorize("@ss.hasPermi('system:device:bindSn')")
    @PutMapping("/{deviceId}/bindSn/{sn}")
    public AjaxResult bindSn(@PathVariable @ApiParam("设备ID") Long deviceId, @PathVariable @ApiParam("SN") String sn) {
        return success(smDeviceService.bindSn(deviceId, sn));
    }

    @ApiOperation("设备充值时长")
    @PreAuthorize("@ss.hasPermi('system:device:addTime')")
    @PutMapping("/addTime/{deviceId}")
    public AjaxResult addTime(@PathVariable @ApiParam("设备id") Long deviceId, @ApiParam("时长（分）") Long amount) {
        return toAjax(smDeviceService.addTimeByUser(deviceId, amount * 60, true, "管理员手动充值"));
    }

    @ApiOperation("设备时长归零")
    @PreAuthorize("@ss.hasPermi('system:device:reset')")
    @PutMapping("/{deviceId}/reset")
    public AjaxResult reset(@PathVariable @ApiParam("设备id") Long deviceId) {
        return success(smDeviceService.reset(deviceId));
    }

    @ApiOperation("设备开关")
    @PreAuthorize("@ss.hasPermi('system:device:switch')")
    @PutMapping("/{deviceId}/switch")
    public AjaxResult switchDevice(@PathVariable Long deviceId, @RequestParam Boolean open) {
        return toAjax(smDeviceService.switchDevice(deviceId, open));
    }

    @ApiOperation("设备批量修改型号")
    @PreAuthorize("@ss.hasPermi('system:device:edit')")
    @PutMapping("/batchUpdateModel")
    public AjaxResult batchUpdateModel(@RequestBody @Validated DeviceBatchUpdateModelDTO dto) {
        return success(smDeviceService.batchUpdateModel(dto));
    }

    @ApiOperation("解除设备绑定")
    @PreAuthorize("@ss.hasPermi('system:device:unbind')")
    @DeleteMapping("/{deviceId}/unbind")
    public AjaxResult unbind(@PathVariable @ApiParam("设备id") Long deviceId) {
        return toAjax(smDeviceService.unbind(deviceId));
    }
}
